/*
 * @(#)server.c	1.3 96/08/08
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*
 * Optional native code support for server base.
 */

#include <pwd.h>
#include <grp.h>
#include <errno.h>
#include <stdio.h>

#include "sun_server_Server.h"

long
sun_server_Server_setUser(Hsun_server_Server *Hthis, HString *Huser)
{
    if (Huser == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return FALSE;
    } else {
	char buf[128];
	char *s = makeCString(Huser);
	if (isdigit(s[0])) {
	    if (setuid(atoi(s)) == -1) {
		if (errno == EINVAL) {
		    jio_snprintf(buf, sizeof(buf), "Invalid user id: %s", s);
		    SignalError(0, JAVAPKG "IllegalArgumentException", buf);
		}
		return FALSE;
	    }
	} else {
	    struct passwd *ent = getpwnam(s);
	    if (ent == 0) {
		jio_snprintf(buf, sizeof(buf), "Invalid user: %s", s);
		SignalError(0, JAVAPKG "IllegalArgumentException", buf);
		return FALSE;
	    } else if (setuid(ent->pw_uid) == -1) {
		return FALSE;
	    }
	}
    }
    return TRUE;
}

long
sun_server_Server_setGroup(Hsun_server_Server *Hthis, HString *Hgroup)
{
    if (Hgroup == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return FALSE;
    } else {
	char buf[128];
	char *s = makeCString(Hgroup);
	if (isdigit(s[0])) {
	    if (setgid(atoi(s)) == -1) {
		if (errno == EINVAL) {
		    jio_snprintf(buf, sizeof(buf), "Invalid group id: %s", s);
		    SignalError(0, JAVAPKG "IllegalArgumentException", buf);
		}
		return FALSE;
	    }
	} else {
	    struct group *ent = getgrnam(s);
	    if (ent == 0) {
		jio_snprintf(buf, sizeof(buf), "Invalid group: %s", s);
		SignalError(0, JAVAPKG "IllegalArgumentException", buf);
		return FALSE;
	    } else if (setgid(ent->gr_gid) == -1) {
		return FALSE;
	    }
	}
    }
    return TRUE;
}
